package plg.overlay;/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

import com.intellij.openapi.Disposable;
import com.intellij.openapi.editor.Editor;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Created by Peter on 01.02.14.
 * Actual Swing container for all Icons above the Editor.
 */
public class OverlayContainerImpl extends JComponent implements Disposable, OverlayContainer {
    private final Map<OverlayElement, OverlayComponent> myItems = new ConcurrentHashMap<>();
    //fps variables
    private final List<Long> updateTimes = new ArrayList<>();
    @NotNull
    private final Editor editor;
    private long lastTime = System.currentTimeMillis();
    private int frameCount = 0;

    public OverlayContainerImpl(@NotNull final Editor editor) {
        super();
        this.editor = editor;

        this.setLayout(null); //use absolute positioning
        editor.getContentComponent().add(this);
        this.setBounds(0, 0, editor.getContentComponent().getWidth(), editor.getContentComponent().getHeight());
        this.setSize(editor.getContentComponent().getWidth(), editor.getContentComponent().getHeight());
        //keep this Container the same size as the editor
        @NotNull ComponentAdapter componentAdapter = new ComponentAdapter() {
            @Override
            public void componentResized(@NotNull ComponentEvent e) {
                super.componentResized(e);
                OverlayContainerImpl.this.setSize(e.getComponent().getWidth(), e.getComponent().getHeight());
                repaintAllIcons();
            }
        };
        editor.getContentComponent().addComponentListener(componentAdapter);
    }

    @Override
    public Dimension getCurrentContainerSize() {
        return this.getSize();
    }

    private void calculateFrameRate() {
        long time = System.currentTimeMillis();
        updateTimes.add(time);
        // We will have the wrong framerate for the first 30 draws. No big.
        //System.out.println("Fps: "+ 1000f / (time-lastTime));
        frameCount++;
        if (time - lastTime > 1000) {
            float timeInSec = (time - updateTimes.get(0)) / 1000f;
            System.out.println("Fps: " + (1000f * frameCount / (time - lastTime))); //NON-NLS
            lastTime = time;
            frameCount = 0;
        }
        if (updateTimes.size() == 31) {
            //float timeInSec = (time - updateTimes.get(0)) / 1000f;
            //System.out.println(30f / timeInSec);
            updateTimes.remove(0);
        }
    }

    @NotNull
    @Override
    public Editor getEditor() {
        return editor;
    }

    /**
     * Adds a new plg.overlay.OverlayElementImpl to this container. No checking if the Document is the same as in this editor.
     */
    @Override
    public void addOverlayElement(@NotNull OverlayElement element) {
        if (myItems.containsKey(element)) {
            return;
        }
        @NotNull OverlayComponent comp = new OverlayComponent(element, this);
        myItems.put(element, comp);
        this.add(comp);
        comp.setVisible(true);
        comp.setEnabled(true);
        comp.repaint();
    }

    @Override
    public void removeOverlayElement(@NotNull final OverlayElement element) {
        UIUtil.invokeLaterIfNeeded(new Runnable() {
            @Override
            public void run() {
                synchronized (myItems) {
                    OverlayComponent overlayComponent = myItems.get(element);
                    if (overlayComponent == null) {
                        return;
                    }
                    overlayComponent.setVisible(false);
                    overlayComponent.setEnabled(false);
                    OverlayContainerImpl.this.remove(overlayComponent);
                    myItems.remove(element);
                }
            }
        });

    }


    @NotNull
    @Override
    public Collection<OverlayElement> getOverlayElements() {
        synchronized (myItems) {
            return new ArrayList<>(myItems.keySet());
        }
    }


    /**
     * Rechecks the internal cache and removes all Icons that have invalid OverlayElements.
     */
    @Override
    public void revalidateAllIcons() {
        synchronized (myItems) {
            for (@NotNull OverlayElement element : myItems.keySet()) {
                if (!element.isValid()) {
                    removeOverlayElement(element);
                }
            }
        }

    }

    /**
     * Calls repaint on all contained Icons, for example after resizing of the editor window.
     */
    @Override
    public void repaintAllIcons() {
        for (@NotNull OverlayComponent overlayComponent : myItems.values()) {
            overlayComponent.repaint();
        }
    }


    @Override
    public void paint(Graphics g) {
        super.paint(g);
        for (OverlayComponent overlayComponent : myItems.values()) {
            overlayComponent.updateLocationAndSize();
        }
//        int size = this.getComponentCount();
//        System.out.println("Overlay: "+size+" "+myItems.size());
//        for (Component component : this.getComponents()) {
//            System.out.println("\t"+component.isEnabled()+" "+component.isVisible()+ " "+ component.getLocation()+" "+component.getSize());
//        }

    }


    @Override
    public void dispose() {
        myItems.clear();
        for (@NotNull Component component : getComponents()) {
            component.setVisible(false);
            component.setEnabled(false);
        }
        this.removeAll();
        assert (this.getComponentCount() == 0);
    }

    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        @NotNull OverlayContainerImpl that = (OverlayContainerImpl) o;

        if (editor != null ? !editor.equals(that.editor) : that.editor != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return editor != null ? editor.hashCode() : 0;
    }
}
