/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.overlay;

import com.intellij.openapi.actionSystem.DataKey;
import com.intellij.openapi.actionSystem.DataProvider;
import com.intellij.openapi.editor.Editor;
import com.intellij.psi.PsiElement;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.awt.*;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSourceAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeListener;

/**
 * Created by Peter on 19.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public interface OverlayElement<T extends PsiElement> extends DataProvider {
    public static final DataKey<OverlayElement> OVERLAY_ELEMENT_DATA_KEY = DataKey.create("sketchlink.overlayElement");


    /**
     * Gets the current alignment of the icon in relation to the PositionalElement.
     */
    @NotNull
    public IconAlignment getIconAlignment();

    /**
     * Sets the alignment of the icon in relation to the PositionalElement.
     */
    public void setIconAlignment(@NotNull IconAlignment iconAlignment);


    @Nullable
    public DragSourceAdapter getDragsourceAdapter();

    /**
     * Sets a DragSourceAdapter, allows the overlay to support dragging
     */
    public void setDragsourceAdapter(@Nullable DragSourceAdapter dragsourceAdapter);


    @Nullable
    public TransferHandler getTransferHandler();

    /**
     * Sets a TransferHandler, needed for Drag and/or drop support.
     */
    public void setTransferHandler(@Nullable TransferHandler transferHandler);

    @Nullable
    public DragGestureListener getDragGestureListener();

    /**
     * Sets a DragGestureListener, for Drag support.
     */
    public void setDragGestureListener(@Nullable DragGestureListener dragGestureListener);

    /**
     * Gets the Mouse cursor that is to be used when the mouse is over this icon.
     */
    @NotNull
    public Cursor getMouseCursor();

    /**
     * Sets the mouse cursor of the mouse, when it is over this overlay
     */
    public void setMouseCursor(@NotNull Cursor mouseCursor);

    @Nullable
    public MouseListener getMouseListener();

    /**
     * Sets the mouseListener that is to be used for this icon.
     * This MouseListener should support several different overlays due to intelliJ's split-view feature.
     */
    public void setMouseListener(@Nullable MouseListener mouseListener);

    @Nullable
    public MouseMotionListener getMouseMotionListener();

    /**
     * Sets the mousemostionlistener used for this overlay icon.
     * Should support several different overlays (e.g. recheck event.getComponent() every time),
     * due to IntelliJ's split-view feature.
     */
    public void setMouseMotionListener(@Nullable MouseMotionListener mouseMotionListener);

    /**
     * Gets the PsiElement that describes the location of the overlay element in general.
     * If null, the Overlay icon will be automatically removed from the editor.
     */
    @Nullable
    public T getPositionalElement();

    /**
     * Sets the PsiElement that describes the location of the overlay element.
     */
    public void setPositionalElement(@NotNull T element);


    /**
     * Gets the ImageIcon that is drawn above the editor.
     * This method may be called several times before drawing once, so try to avoid lot of computation.
     */
    @NotNull
    public ImageIcon getImageIcon();

    /**
     * Sets the ImageIcon that is drawn above the editor.
     */
    public void setImageIcon(@NotNull ImageIcon icon);


    public void addPropertyChangeListener(@NotNull PropertyChangeListener listener);

    public void removePropertyChangeListener(@NotNull PropertyChangeListener listener);

    /**
     * Return whether this icon should be drawn. If not valid, it is skipped and this OverlayElementImpl may be unregistered.
     */
    public boolean isValid();

    public void setValid(boolean valid);


    /**
     * Gets the actual offset that describes the location of this OverlayElementImpl.
     * Only values between 0 and document.length are valid.
     */
    public int getOffset(@NotNull Editor editor);


    /**
     * Gets the default Image size that getImageIcon is supposed to be.
     */
    @NotNull
    public Dimension getPreferredImageSize();

    /**
     * Sets the default image size that getImageIcon is supposed to be.
     * If it differs from the returned size, the imageIcon is scaled.
     * Important for the editor's presentation and fullscreen mode.
     */
    public void setPreferredImageSize(@NotNull Dimension preferredImageSize);
}
